@@ -6,19 +6,20 @@ from pathlib import Path
 from dataclasses import dataclass, asdict, field
 from typing import Optional, List, Any, Callable, TypeVar
 from abc import ABC, abstractmethod
+from functools import lru_cache
 
 from .shared import *
 
 TZ = datetime.timezone(offset=datetime.timedelta(seconds=time.timezone))
 
 
-def parse_datetime(s: str) -> datetime.datetime:
+def deserialize_datetime(s: str) -> datetime.datetime:
     """Deserialize our standard format."""
 
     return datetime.datetime.strptime(s, "%Y-%m-%d %H:%M:%S").replace(tzinfo=TZ)
 
 
-def dump_datetime(d: datetime) -> str:
+def serialize_datetime(d: datetime) -> str:
     """Serialize back."""
 
     return d.strftime("%Y-%m-%d %H:%M:%S")
@@ -131,26 +132,29 @@ class ProblemGrading(Model):
         return self.manual is not None and self.manual.enabled and self.problem.grading.enabled
 
     def percentage(self) -> Decimal:
+        """Percentage weight of the problem in the assignment."""
+
         return self.weight / self.problem.assignment.grading.weight()
 
     @property
-    def _category_weight(self) -> Decimal:
+    @lru_cache(maxsize=1)
+    def weight_total(self) -> Decimal:
         return sum((
             self.automated.weight if self.automated and self.automated.enabled else 0,
             self.review.weight if self.review and self.review.enabled else 0,
             self.manual.weight if self.manual and self.manual.enabled else 0))
 
     @property
-    def points_automated(self) -> Decimal:
-        return self.automated.weight / self._category_weight * self.points
+    def percentage_automated(self) -> Decimal:
+        return self.automated.weight / self.weight_total
 
     @property
-    def points_review(self) -> Decimal:
-        return self.review.weight / self._category_weight * self.points
+    def percentage_review(self) -> Decimal:
+        return self.review.weight / self.weight_total
 
     @property
-    def points_manual(self) -> Decimal:
-        return self.manual.weight / self._category_weight * self.points
+    def percentage_manual(self) -> Decimal:
+        return self.manual.weight / self.weight_total
 
     @classmethod
     def load(cls, data: dict, problem: "Problem" = None) -> "ProblemGrading":
@@ -209,12 +213,15 @@ class Problem(Model):
     def load(cls, data: dict, assignment: "Assignment" = None) -> "Problem":
         """Load directly from a dictionary."""
 
+        # Get override types
+        problem_grading_type = cls.__annotations__.get("grading", ProblemGrading)
+
         self = cls(
             assignment=assignment,
             short=data["short"],
             title=data["title"],
             relative_path=Path(data["relative_path"]),
-            grading=ProblemGrading.load(data["grading"]),
+            grading=problem_grading_type.load(data["grading"]),
             authors=list(map(Author.load, data["authors"])),
             topics=data["topics"],
             notes=data.get("notes"),
@@ -248,15 +255,15 @@ class AssignmentDates(Model):
         """Convert to datetime."""
 
         return cls(
-            assigned=parse_datetime(data["assigned"]),
-            due=parse_datetime(data["due"]),)
+            assigned=deserialize_datetime(data["assigned"]),
+            due=deserialize_datetime(data["due"]),)
 
     def dump(self) -> dict:
         """Specifically serialize the datetime."""
 
         return dict(
-            assigned=dump_datetime(self.assigned),
-            due=dump_datetime(self.due),)
+            assigned=serialize_datetime(self.assigned),
+            due=serialize_datetime(self.due),)
 
 
 @dataclass(eq=False)
@@ -266,22 +273,17 @@ class AssignmentGrading(Model):
     points: int
     assignment: "Assignment" = field(default=None)
 
-    _weight: Decimal = field(init=False)
-
     @classmethod
     def load(cls, data: dict, assignment: "Assignment" = None) -> "AssignmentGrading":
         """Load from serialized."""
 
         return cls(points=data["points"], assignment=assignment)
 
+    @lru_cache(maxsize=1)
     def weight(self) -> Decimal:
         """Compute cumulative weight of all problems."""
 
-        try:
-            return self._weight
-        except AttributeError:
-            self._weight = sum(problem.grading.weight for problem in self.assignment.problems)
-            return self._weight
+        return sum(problem.grading.weight for problem in self.assignment.problems)
 
     def dump(self) -> dict:
         """Avoid recursion."""
@@ -301,14 +303,14 @@ class AssignmentMeta(Model):
         """Deserialize datetime."""
 
         return cls(
-            built=parse_datetime(data["built"]),
+            built=deserialize_datetime(data["built"]),
             curricula=data["curricula"],)
 
     def dump(self) -> dict:
         """Serialize the datetime here too."""
 
         return dict(
-            built=dump_datetime(self.built),
+            built=serialize_datetime(self.built),
             curricula=version,)
 
 
@@ -320,8 +322,10 @@ class Assignment(Model):
     title: str
     authors: List[Author]
     dates: AssignmentDates
+
     problems: List[Problem]
     grading: AssignmentGrading
+
     notes: Optional[str] = None
     meta: AssignmentMeta = AssignmentMeta()
 
